<?php
/*
Plugin Name: Shortcode List (EasyToolsWP)
Description: A plugin that automatically displays all shortcodes found in plugins and themes.
Version: 1.1
Author: EasyToolsWP
*/

// Function to find all shortcodes in plugins and themes
function find_all_shortcodes() {
    global $shortcode_tags;
    $shortcodes = array();

    foreach ($shortcode_tags as $tag => $func) {
        $reflection = null;
        $parameters = [];
        $plugin_name = 'Unknown';
        $callback_info = '';

        // Attempt to get the plugin name by searching active plugins
        foreach (get_plugins() as $plugin_file => $plugin_data) {
            if (is_plugin_active($plugin_file)) {
                if (has_shortcode_in_plugin($plugin_file, $tag)) {
                    $plugin_name = $plugin_data['Name'];
                    break;
                }
            }
        }

        // Use reflection to find the callback and parameters
        if (is_array($func) && is_callable($func)) {
            try {
                $reflection = new ReflectionMethod($func[0], $func[1]);
                $callback_info = get_class($func[0]) . '::' . $func[1];
            } catch (ReflectionException $e) {
                // Handle reflection exception if necessary
            }
        } elseif (is_string($func) && function_exists($func)) {
            try {
                $reflection = new ReflectionFunction($func);
                $callback_info = $func;
            } catch (ReflectionException $e) {
                // Handle reflection exception if necessary
            }
        }

        if ($reflection) {
            foreach ($reflection->getParameters() as $param) {
                $parameters[] = $param->name;
            }
        }

        $shortcodes[] = array(
            'tag' => $tag,
            'callback' => $callback_info,
            'parameters' => $parameters,
            'plugin_name' => $plugin_name,
            'example' => generate_example_shortcode($tag, $parameters)
        );
    }

    return $shortcodes;
}

// Function to generate an example of a shortcode
function generate_example_shortcode($tag, $parameters) {
    $shortcode_example = '';
    
    // Example attributes for the shortcode
    $atts = shortcode_atts(array(
        'lang' => 'php',
    ), array(), $tag);

    // Build an example based on attributes
    $shortcode_example = '[' . $tag;
    foreach ($atts as $key => $value) {
        $shortcode_example .= ' ' . esc_attr($key) . '="' . esc_attr($value) . '"';
    }
    $shortcode_example .= ']';

    // Add content if relevant
    if (!empty($parameters)) {
        $shortcode_example .= '...[/'.$tag.']';
    }
    
    return $shortcode_example;
}

// Function to check if a shortcode exists in a plugin
function has_shortcode_in_plugin($plugin_file, $shortcode_tag) {
    // Temporarily include plugin file to extract shortcodes
    $plugin_path = WP_PLUGIN_DIR . '/' . $plugin_file;
    include_once($plugin_path);

    // Find shortcodes registered by the plugin
    global $shortcode_tags;
    return isset($shortcode_tags[$shortcode_tag]);
}

// Function to display shortcodes on an admin page
function show_shortcode_list() {
    $shortcodes = find_all_shortcodes();
    echo '<div class="wrap"><h2>List of Shortcodes</h2><table class="widefat"><thead><tr><th>Plugin</th><th>Shortcode</th><th>Example</th><th>Attributes</th><th>Callback</th></tr></thead><tbody>';
    foreach ($shortcodes as $shortcode) {
        echo '<tr><td>' . esc_html($shortcode['plugin_name']) . '</td><td><div class="shortcode-box" data-shortcode="[' . esc_html($shortcode['tag']) . ']">[' . esc_html($shortcode['tag']) . ']</div></td><td>' . esc_html($shortcode['example']) . '</td><td>' . esc_html(implode(', ', $shortcode['parameters'])) . '</td><td>' . esc_html($shortcode['callback']) . '</td></tr>';
    }
    echo '</tbody></table></div>';
    echo '<script>
        document.querySelectorAll(".shortcode-box").forEach(function(box) {
            box.addEventListener("click", function() {
                var shortcode = this.getAttribute("data-shortcode");
                navigator.clipboard.writeText(shortcode).then(function() {
                    alert("Shortcode copied: " + shortcode);
                }, function() {
                    alert("Failed to copy shortcode.");
                });
            });
        });
    </script>';
    echo '<style>
        .shortcode-box {
            display: inline-block;
            padding: 5px 10px;
            margin: 5px 0;
            border: 1px solid #ccc;
            background-color: #f9f9f9;
            cursor: pointer;
        }
        .shortcode-box:hover {
            background-color: #e9e9e9;
        }
    </style>';
}

// Function to add a submenu under "Tools"
function register_shortcode_list_menu() {
    add_submenu_page(
        'tools.php',
        'Shortcode List', 
        'Shortcode List', 
        'manage_options', 
        'shortcode-list', 
        'show_shortcode_list'
    );
}

add_action('admin_menu', 'register_shortcode_list_menu');

// Shortcode to display the shortcode list on the frontend
function display_shortcodes_on_frontend() {
    $shortcodes = find_all_shortcodes();
    $output = '<table class="shortcode-table"><thead><tr><th>Plugin</th><th>Shortcode</th><th>Example</th><th>Attributes</th><th>Callback</th></tr></thead><tbody>';
    foreach ($shortcodes as $shortcode) {
        $output .= '<tr><td>' . esc_html($shortcode['plugin_name']) . '</td><td><div class="shortcode-box" data-shortcode="[' . esc_html($shortcode['tag']) . ']">[' . esc_html($shortcode['tag']) . ']</div></td><td>' . esc_html($shortcode['example']) . '</td><td>' . esc_html(implode(', ', $shortcode['parameters'])) . '</td><td>' . esc_html($shortcode['callback']) . '</td></tr>';
    }
    $output .= '</tbody></table>';
    $output .= '<script>
        document.querySelectorAll(".shortcode-box").forEach(function(box) {
            box.addEventListener("click", function() {
                var shortcode = this.getAttribute("data-shortcode");
                navigator.clipboard.writeText(shortcode).then(function() {
                    alert("Shortcode copied: " + shortcode);
                }, function() {
                    alert("Failed to copy shortcode.");
                });
            });
        });
    </script>';
    $output .= '<style>
        .shortcode-box {
            display: inline-block;
            padding: 5px 10px;
            margin: 5px 0;
            border: 1px solid #ccc;
            background-color: #f9f9f9;
            cursor: pointer;
        }
        .shortcode-box:hover {
            background-color: #e9e9e9;
        }
    </style>';
    return $output;
}

add_shortcode('shortcode_list', 'display_shortcodes_on_frontend');
